
#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
#endregion


// 
//
// ALL Media files must be PART of the solution (to be deployed over to the XBOX)!!


namespace BallDroppingOnXBox
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        ContentManager content;

        private BasicEffect basicEffect;
        private EffectPool effectPool;
        private Matrix worldMatrix, viewMatrix, projectionMatrix;

        // these are the size of the output window, ignored
        // on Xbox 360
        private int preferredWindowWidth = 640;
        private int preferredWindowHeight = 360;

        // these are the size of the offscreen drawing surface
        // in general, no one wants to change these as there
        // are all kinds of UI calculations and positions based
        // on these dimensions.
        const int FixedDrawingWidth = 1280;
        const int FixedDrawingHeight = 720;

        Circle2D myCircle;
        float mySign;
        float rot;
        Texture2D myTexture;

        public Game1()
        {


#if XBOX360
            // we might as well use the xbox in all its glory
            preferredWindowWidth = FixedDrawingWidth;
            preferredWindowHeight = FixedDrawingHeight;
            // enableDrawScaling = false;
#else
            // graphics.enableDrawScaling = true;

            I don't think I support this for now ...
#endif

            graphics = new GraphicsDeviceManager(this);
            graphics.PreferredBackBufferWidth = preferredWindowWidth;
            graphics.PreferredBackBufferHeight = preferredWindowHeight;
            // graphics.PreparingDeviceSettings += new EventHandler<PreparingDeviceSettingsEventArgs>(PreparingDeviceSettings);

            // Game should run as fast as possible.
            IsFixedTimeStep = false;
        }


        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            //  my object
            myCircle = new Circle2D(graphics.GraphicsDevice, new Vector3(5.0f, 3.0f, 0.0f), 1.0f, 100);
            mySign = -1;
            rot = 0.0f;

            // my device settings
            graphics.GraphicsDevice.RenderState.DepthBufferEnable = false;
            graphics.GraphicsDevice.RenderState.MultiSampleAntiAlias = true;
            graphics.GraphicsDevice.RenderState.CullMode = CullMode.None;
                        
            // render "effect"
            effectPool = new EffectPool();
            basicEffect = new BasicEffect(graphics.GraphicsDevice, effectPool);

            // initialize the transformation pipeline to draw in 12.8x7.2 area
            worldMatrix = new Matrix();
            viewMatrix = new Matrix();
            projectionMatrix = new Matrix();

            Matrix s, t;
            t = Matrix.CreateTranslation(-1.0f, -1.0f, 0.0f);
            s = Matrix.CreateScale(2.0f / 12.8f, 2.0f / 7.2f, 1.0f);
            worldMatrix = Matrix.Identity;
            viewMatrix = s * t;
            projectionMatrix = Matrix.Identity;
    
            // here is the Texture bit ...
            basicEffect.World = worldMatrix;
            basicEffect.View = viewMatrix;
            basicEffect.Projection = projectionMatrix;
            basicEffect.LightingEnabled = false;
            basicEffect.VertexColorEnabled = true;  // Gouraud Shading

            base.Initialize();
        }


        /// <summary>
        /// Load your graphics content.  If loadAllContent is true, you should
        /// load content from both ResourceManagementMode pools.  Otherwise, just
        /// load ResourceManagementMode.Manual content.
        /// </summary>
        /// <param name="loadAllContent">Which type of content to load.</param>
        protected override void LoadGraphicsContent(bool loadAllContent)
        {
            base.LoadGraphicsContent(loadAllContent);

            if (loadAllContent)
            {
                content = new ContentManager(Services);

                String name = @"texture\Spacewar";

                // load in the texture
                myTexture = content.Load<Texture2D>(name);
            }

            // TODO: Load any ResourceManagementMode.Manual content
        }


        /// <summary>
        /// Unload your graphics content.  If unloadAllContent is true, you should
        /// unload content from both ResourceManagementMode pools.  Otherwise, just
        /// unload ResourceManagementMode.Manual content.  Manual content will get
        /// Disposed by the GraphicsDevice during a Reset.
        /// </summary>
        /// <param name="unloadAllContent">Which type of content to unload.</param>
        protected override void UnloadGraphicsContent(bool unloadAllContent)
        {
            base.UnloadGraphicsContent(unloadAllContent);
            if (unloadAllContent == true)
            {
                content.Dispose();
            }
        }


        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the default game to exit on Xbox 360 and Windows
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            Vector3 c = myCircle.getCenter();
            c.Y += (0.01f * mySign);

            if (c.Y < 0.0f)
            {
                mySign = 1.0f;
            }
            else
            {
                if (c.Y > 7.2)
                {
                    mySign = -1.0f;
                }
            }
            rot += 0.01f;
            if (rot > (2.0*Math.PI))
                rot -= (float) (2*Math.PI);
            myCircle.setCenter(c);

            Matrix r, t, tb;
            t = Matrix.CreateTranslation(-c.X, -c.Y, 0.0f);
            tb = Matrix.CreateTranslation(c.X, c.Y, 0.0f);
            r = Matrix.CreateRotationZ(rot);
            basicEffect.World = t * r * tb;

            



            base.Update(gameTime);
        }


        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

            // TODO: Add your drawing code here

            base.Draw(gameTime);

            basicEffect.Texture = myTexture;
            basicEffect.TextureEnabled = true;


            // for now, I think the basicEffect is meant to be associated with each Primitive object
            basicEffect.Begin();
            basicEffect.Techniques[0].Passes[0].Begin();
            basicEffect.CommitChanges();

            // do the drawing of each primitives ...
                    myCircle.Draw(graphics.GraphicsDevice);

            basicEffect.Techniques[0].Passes[0].End();
            basicEffect.End();

        }
    }
}